import numpy as np
import matplotlib.pyplot as plt

def walsh_code(n):
    if n == 1:
        return np.array([[1]])
    else:
        previous_code = walsh_code(n // 2)
        top = np.hstack([previous_code, previous_code])
        bottom = np.hstack([previous_code, -previous_code])
        return np.vstack([top, bottom])

def cdma_modulate(data, code):
    return data[:, None] * code

# Data for two users
data_user1 = np.array([1, -1, 1, -1])
data_user2 = np.array([-1, 1, -1, 1])

# Walsh codes for two users
walsh_codes = walsh_code(2)  # Generate 2x2 Walsh matrix

# Modulate data for each user
modulated_signal_user1 = cdma_modulate(data_user1, walsh_codes[0])
modulated_signal_user2 = cdma_modulate(data_user2, walsh_codes[1])

# Combine signals (simulating transmission over the same channel)
combined_signal = modulated_signal_user1 + modulated_signal_user2

# Plot combined signal
plt.figure(figsize=(10, 6))
for i in range(2):
    plt.subplot(2, 1, i+1)
    plt.step(range(len(combined_signal)), combined_signal[:, i], where='mid')
    plt.title(f'Combined Signal on Channel {i+1}')
plt.tight_layout()
plt.show()
