import numpy as np
import matplotlib.pyplot as plt

def generate_ofdm_signal(data, N, cp_len):
    # Generate the OFDM symbol using IFFT
    symbol = np.fft.ifft(data, n=N)
    # Add cyclic prefix
    cp = symbol[-cp_len:]  # Take the last cp_len samples
    ofdm_signal = np.concatenate([cp, symbol])
    return ofdm_signal

# Parameters for OFDM
N = 64  # Number of subcarriers
cp_len = 16  # Cyclic prefix length

# Random BPSK data for OFDM transmission
data = np.random.choice([-1, 1], N)

# Generate OFDM signal
ofdm_signal = generate_ofdm_signal(data, N, cp_len)

# Plot the OFDM signal
plt.figure(figsize=(10, 4))
plt.plot(ofdm_signal.real)
plt.title('Real Part of OFDM Signal')
plt.xlabel('Time')
plt.ylabel('Amplitude')
plt.show()

